<?php

namespace Revolut\Payment\Gateway\Config;

use Magento\Framework\Encryption\Encryptor;
use Revolut\Payment\Model\Source\Mode;
use Magento\Store\Model\ScopeInterface;
use Revolut\Payment\Model\Helper\ConstantValue;
use Revolut\Payment\Model\ResourceModel\CustomConfigModel;
use Magento\Payment\Model\MethodInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Cache\TypeListInterface;
use Magento\Config\Model\ResourceModel\Config as ConfigInterface;

class Config extends \Magento\Payment\Gateway\Config\Config
{
    public const IS_ACTIVE = 'active';
    public const API_MODE = 'api_mode';
    public const API_KEY_LIVE = 'api_key_live';
    public const API_KEY_SANDBOX = 'api_key_sandbox';
    public const PAYMENT_ACTION = 'payment_action';
    public const KEY_TITLE = 'title';
    public const WEBHOOK_SETUP = 'webhook_setup';
    public const APPLEPAY_DOMAIN_SETUP = 'apple_pay_domain_setup';
    public const CONFIG_CODE = 'payment/revolut_card/';

    /**
     * @var Encryptor
     */
    protected $encryptor;

    /**
     * @var ConfigInterface
     */
    protected $configInterface;
    
    /**
     * @var CustomConfigModel
     */
    protected $customConfigModel;

    /**
     * @var TypeListInterface
     */
    protected $cacheTypeList;

    /**
     * @param ScopeConfigInterface $scopeConfig
     * @param Encryptor $encryptor
     * @param ConfigInterface $configInterface
     * @param CustomConfigModel $customConfigModel
     * @param TypeListInterface $cacheTypeList
     * @param string|null $methodCode
     * @param string $pathPattern
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        Encryptor $encryptor,
        ConfigInterface $configInterface,
        CustomConfigModel $customConfigModel,
        TypeListInterface $cacheTypeList,
        $methodCode = null,
        $pathPattern = self::DEFAULT_PATH_PATTERN
    ) {
        parent::__construct($scopeConfig, $methodCode, $pathPattern);
        
        $this->encryptor = $encryptor;
        $this->configInterface = $configInterface;
        $this->customConfigModel = $customConfigModel;
        $this->cacheTypeList = $cacheTypeList;
    }

    /**
     * Check if gateway enabled
     *
     * @param  int $storeId
     * @return bool
     */
    public function isActive($storeId = 0)
    {
        return (bool) $this->getValue(self::IS_ACTIVE, $storeId);
    }
    
    /**
     * Get Webhook Setup Result
     *
     * @param  int $storeId
     * @return mixed
     */
    public function getWebhookSetupResult($storeId)
    {
        return $this->customConfigModel->getConfig(
            self::CONFIG_CODE . self::WEBHOOK_SETUP . '_' . $this->getModeName($storeId),
            ScopeInterface::SCOPE_STORE,
            $storeId
        );
    }
    
    /**
     * Get Webhook Setup Result
     *
     * @param  int $storeId
     * @return mixed
     */
    public function getApplePayDomainSetupResult($storeId = 0)
    {
        return $this->customConfigModel->getConfig(
            self::CONFIG_CODE . self::APPLEPAY_DOMAIN_SETUP,
            ScopeInterface::SCOPE_STORE,
            $storeId
        );
    }
    
    /**
     * Save Webhook Setup Result
     *
     * @param  int $result
     * @param  int $storeId
     * @return void
     */
    public function saveWebhookSetupResult($result, $storeId)
    {
        $this->configInterface->saveConfig(
            self::CONFIG_CODE . self::WEBHOOK_SETUP . '_' . $this->getModeName($storeId),
            (string)$result,
            ScopeInterface::SCOPE_STORE,
            $storeId
        );
        $this->cacheTypeList->cleanType(\Magento\Framework\App\Cache\Type\Config::TYPE_IDENTIFIER);
    }
    
    /**
     * Save Webhook Setup Result
     *
     * @param string $result
     * @param int $storeId
     * @return void
     */
    public function saveApplePayDomainSetupResult($result, $storeId = 0)
    {
        $this->configInterface->saveConfig(
            self::CONFIG_CODE . self::APPLEPAY_DOMAIN_SETUP,
            (string)$result,
            ScopeInterface::SCOPE_STORE,
            $storeId
        );
        $this->cacheTypeList->cleanType(\Magento\Framework\App\Cache\Type\Config::TYPE_IDENTIFIER);
    }
    
    /**
     * Get title of payment
     *
     * @param int $storeId
     * @return string
     */
    public function getTitle($storeId = 0)
    {
        return (string) $this->getValue(self::KEY_TITLE, $storeId);
    }
    
    /**
     * Get payment action
     *
     * @param int|null $storeId
     * @return mixed
     */
    public function getPaymentAction($storeId)
    {
        return $this->getValue(self::PAYMENT_ACTION, $storeId);
    }

    /**
     * Check if payment action is manual
     *
     * @param int $storeId
     * @return bool
     */
    public function isManualCapture($storeId = 0)
    {
        return $this->getValue(self::PAYMENT_ACTION, $storeId) != MethodInterface::ACTION_AUTHORIZE_CAPTURE;
    }
    
    /**
     * Check if payment action is automatic
     *
     * @param int $storeId
     * @return bool
     */
    public function isAutomaticCapture($storeId = 0)
    {
        return $this->getValue(self::PAYMENT_ACTION, $storeId) == MethodInterface::ACTION_AUTHORIZE_CAPTURE;
    }
    
    /**
     * Check Payment configuration mode is production
     *
     * @param int $storeId
     * @return bool
     */
    public function isLiveMode($storeId = 0)
    {
        return $this->getValue(self::API_MODE, $storeId) == Mode::PRODUCTION_MODE;
    }
    
    /**
     * Get Payment configuration mode name
     *
     * @param int $storeId
     * @return string
     */
    public function getModeName($storeId = 0)
    {
        return $this->isLiveMode($storeId) ? ConstantValue::PROD_MODE : ConstantValue::SANDBOX_MODE;
    }
    
    /**
     * Get JS SDK url
     *
     * @param int $storeId
     * @return string
     */
    public function getSdkUrl($storeId = 0)
    {
        return ConstantValue::REVOLUT_SDK_URLS[$this->getModeName($storeId)];
    }

    /**
     * Get api key
     *
     * @param int $storeId
     * @return mixed
     */
    public function getApiKey($storeId = 0)
    {
        if ($this->isLiveMode($storeId)) {
            return $this->getLiveApiKey($storeId);
        }

        return $this->getSandboxApiKey($storeId);
    }
    
    /**
     * Check api key
     *
     * @param int $storeId
     * @return bool
     */
    public function isApiKeyConfigured($storeId = 0)
    {
        return !empty($this->getApiKey($storeId));
    }
    
    /**
     * Get api key
     *
     * @param int $storeId
     * @return mixed
     */
    public function getLiveApiKey($storeId = 0)
    {
        $value = (string) $this->getValue(self::API_KEY_LIVE, $storeId);
        return $value ? $this->encryptor->decrypt($value) : $value;
    }
    
    /**
     * Get sandbox api key
     *
     * @param int $storeId
     * @return mixed
     */
    public function getSandboxApiKey($storeId = 0)
    {
        $value = (string) $this->getValue(self::API_KEY_SANDBOX, $storeId);
        return $value ? $this->encryptor->decrypt($value) : $value;
    }

    /**
     * Get api url
     *
     * @param int $storeId
     * @return string
     */
    public function getApiUrl($storeId = 0)
    {
        if ($this->isLiveMode($storeId)) {
            return ConstantValue::URL_PROD;
        }
        return ConstantValue::URL_SANDBOX;
    }

     /**
      * @param int $storeId
      * @return string
      */
    public function getMgmtApiUrl($storeId = 0)
    {
        if ($this->isLiveMode($storeId)) {
            return ConstantValue::MGMT_URL_PROD;
        }
        return ConstantValue::MGMT_URL_SANDBOX;
    }
}
