<?php

namespace Revolut\Payment\Model\Api;

use http\Env\Request as Req;
use Revolut\Payment\Api\OrderManagementInterface;
use Revolut\Payment\Model\Helper\Logger;
use Revolut\Payment\Model\Helper\Api\RevolutApi;
use Revolut\Payment\Gateway\Helper\AmountProvider;
use Revolut\Payment\Model\RevolutOrder;
use Revolut\Payment\Model\Helper\ConstantValue;
use Revolut\Payment\Model\RevolutPaymentInformationManagement;
use Revolut\Payment\Model\ResourceModel\RevolutOrder\Collection;
use Revolut\Payment\Model\RevolutOrderFactory;
use Magento\Quote\Api\Data\PaymentInterface;
use Magento\Quote\Api\CartRepositoryInterface;
use Magento\Quote\Api\CartManagementInterface;
use Revolut\Payment\Model\ChangeQuoteControl;

use Magento\Store\Model\StoreManagerInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Json\Helper\Data;
use Magento\Checkout\Model\Session;
use Magento\Customer\Model\Session as CustomerSession;
use Magento\Framework\Webapi\Rest\Request;
use Magento\Quote\Model\Quote;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Quote\Model\QuoteFactory;
use Magento\Framework\Event\ManagerInterface as EventManager;

/**
 * Class OrderManagement
 */
class OrderManagement implements OrderManagementInterface
{
    /**
     * @var AmountProvider
     */
    protected $amountProvider;
    
    /**
     * @var RevolutApi
     */
    protected $revolutApi;

    /**
     * @var Data
     */
    protected $jsonHelper;

    /**
     * @var Session
     */
    protected $checkoutSession;
    
    /**
     * @var CustomerSession
     */
    protected $customerSession;

    /**
     * @var Request
     */
    protected $request;

    /**
     * @var RevolutOrder
     */
    protected $revolutOrder;
    
    /**
     * @var Collection
     */
    protected $revolutOrderCollection;

    /**
     * @var RevolutPaymentInformationManagement
     */
    protected $revolutPaymentInformationManagement;

    /**
     * @var Logger
     */
    protected $logger;
    
    /**
     * @var OrderInterface
     */
    private $orderInterface;

    /**
     * @var EventManager
     */
    protected $eventManager;

    /**
     * @var QuoteFactory
     */
    protected $quoteFactory;

    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;
    
    /**
     * @var RevolutOrderFactory
     */
    protected $revolutOrderFactory;

    /**
     * @var CartRepositoryInterface
     */
    private $cartRepository;

    /**
     * @var CartManagementInterface
     */
    private $cartManagement;

    /**
     * @var ChangeQuoteControl
     */
    private $changeQuoteControl;

    /**
     * @param AmountProvider $amountProvider
     * @param Data $jsonHelper
     * @param RevolutApi $revolutApi
     * @param CustomerSession $customerSession
     * @param Session $checkoutSession
     * @param RevolutOrder $revolutOrder
     * @param Collection $revolutOrderCollection
     * @param OrderInterface $orderInterface
     * @param Request $request
     * @param Logger $logger
     * @param RevolutPaymentInformationManagement $revolutPaymentInformationManagement
     * @param EventManager $eventManager
     * @param QuoteFactory $quoteFactory
     * @param StoreManagerInterface $storeManager
     * @param RevolutOrderFactory $revolutOrderFactory
     * @param CartRepositoryInterface $cartRepository
     * @param CartManagementInterface $cartManagement
     * @param ChangeQuoteControl $changeQuoteControl
     */
    public function __construct(
        AmountProvider $amountProvider,
        Data $jsonHelper,
        RevolutApi $revolutApi,
        CustomerSession $customerSession,
        Session $checkoutSession,
        RevolutOrder $revolutOrder,
        Collection $revolutOrderCollection,
        OrderInterface $orderInterface,
        Request $request,
        Logger $logger,
        RevolutPaymentInformationManagement $revolutPaymentInformationManagement,
        EventManager $eventManager,
        QuoteFactory $quoteFactory,
        StoreManagerInterface $storeManager,
        RevolutOrderFactory $revolutOrderFactory,
        CartRepositoryInterface $cartRepository,
        CartManagementInterface $cartManagement,
        ChangeQuoteControl $changeQuoteControl
    ) {
        $this->revolutApi = $revolutApi;
        $this->amountProvider = $amountProvider;
        $this->jsonHelper = $jsonHelper;
        $this->checkoutSession = $checkoutSession;
        $this->customerSession = $customerSession;
        $this->request = $request;
        $this->revolutOrder = $revolutOrder;
        $this->revolutOrderCollection = $revolutOrderCollection;
        $this->orderInterface = $orderInterface;
        $this->logger = $logger;
        $this->revolutPaymentInformationManagement = $revolutPaymentInformationManagement;
        $this->eventManager = $eventManager;
        $this->quoteFactory = $quoteFactory;
        $this->storeManager = $storeManager;
        $this->revolutOrderFactory = $revolutOrderFactory;
        $this->cartRepository = $cartRepository;
        $this->cartManagement = $cartManagement;
        $this->changeQuoteControl = $changeQuoteControl;
    }

    /**
     * Get or Create Magento Quote
     *
     * @return Quote
     */
    public function getOrCreateQuote()
    {
        $quote = $this->checkoutSession->getQuote();

        if (!$quote->getId()) {
            $this->changeQuoteControl->setAllowed();
            $customer = $this->customerSession->getCustomer();
            $customerId = $customer->getId();
            if ($customerId) {
                $quoteId = $this->cartManagement->createEmptyCartForCustomer($customerId); // @phpstan-ignore-line
            } else {
                $quoteId = $this->cartManagement->createEmptyCart();
            }
            
            $quote = $this->cartRepository->get($quoteId);
            $this->checkoutSession->replaceQuote($quote); // @phpstan-ignore-line
            $quote = $this->checkoutSession->getQuote();
        }

        return $quote;
    }
    
    /**
     * Get or Create Magento Quote
     *
     * @return String
     */
    public function getPublicIdParam()
    {
        $params = $this->request->getBodyParams();
        $publicId = $params['revolut_public_id'];

        if (empty($publicId)) {
            throw new LocalizedException(__('publicId can not be empty'));
        }
        
        return $publicId;
    }

    /**
     * Create order
     *
     * @return array result.
     * @api
     */
    public function create()
    {
        try {
            $quote = $this->getOrCreateQuote();
            $customer = $this->customerSession->getCustomer();
            $quoteId = $quote->getId();
            $customerId = $customer->getId();

            if (empty($quoteId)) {
                throw new LocalizedException(__('Can not load shopping cart'));
            }

            $storeId = $this->storeManager->getStore()->getId();
            $currency = $this->storeManager->getStore()->getCurrentCurrency();
           
            $quote->setStoreId($storeId);
            $quote->setQuoteCurrencyCode($currency->getCode());

            $amount = $quote->getGrandTotal();
            $currency = $quote->getQuoteCurrencyCode();
            $storeId = $quote->getStoreId();
            
            if (empty($currency) || empty($storeId) || empty($amount)) {
                throw new LocalizedException(__('Can not load shopping cart'));
            }

            $amount = $this->amountProvider->convert($amount, $currency);

            $this->logger->debug('create order: ' . $amount . ' ' . $currency . ' storeId: ' . $storeId . ' customerId: ' . $customerId);
            
            $revolutOrderModel = $this->revolutOrderCollection->getOrderByQuoteAndStore($quoteId, $storeId)->getFirstItem(); // @phpstan-ignore-line
            $revolutOrder = $revolutOrderModel->create(['amount' => $amount, 'currency'=> $currency], $quoteId, $customerId, $storeId); // @phpstan-ignore-line
            
            if (!$revolutOrder) {
                $revolutOrderModel = $this->revolutOrderFactory->create();
                $revolutOrder = $revolutOrderModel->create(['amount' => $amount, 'currency'=> $currency], $quoteId, $customerId, $storeId); // @phpstan-ignore-line
            }
            
            $result = [
                'success' => true,
                'customerId' => $customerId,
                'quoteId' => $quoteId,
                'public_id' => $revolutOrder->getPublicId(),
                'currency' => $currency,
                'storeId' => $storeId,
                'amount' => $amount,
                'public_key' => $this->revolutApi->getMerchantPublicKey($storeId),
                'available_card_brands' => $this->revolutApi->getAvailableCardBrands($storeId, $revolutOrder->getPublicId())
            ];
        } catch (\Exception $e) {
            $this->logger->debug('An error occurred while processing the order: ' . $e->getMessage());
            $result = [
                'error' => true,
                'mess' => 'An error occurred while processing the order: ' . $e->getMessage()
            ];
        }

        return [$result];
    }

    /**
     * Handle webhook callback
     *
     * @return array|bool
     * @api
     */
    public function webhook()
    {
        $response = $this->request->getContent();
        $this->logger->debug('start webhook call: ' . json_encode($response));

        if (empty($response)) {
            return ['empty request'];
        }

        $response = (array)$this->jsonHelper->jsonDecode($response);

        if (empty($response['order_id']) || !isset($response['event']) || !in_array($response['event'], ConstantValue::WEBHOOK_EVENTS)) {
            return [];
        }

        try {
            $this->revolutOrder = $this->revolutOrder->retrieveBy('revolut_order_id', $response['order_id']);
            if (empty($this->revolutOrder->revolutOrder['state']) || $this->revolutOrder->revolutOrder['state'] == 'PENDING') {
                return ['payment is not completed : ' . $response['order_id']];
            }
        } catch (\Exception $e) {
            return ['payment not found: ' . $response['order_id']];
        }

        if ($response['event'] == ConstantValue::WEBHOOK_EVENT_ORDER_AUTHORISED && empty($this->revolutOrder->getIncrementOrderId())) {
            $result = $this->revolutPaymentInformationManagement->placeOrder($this->revolutOrder->getPublicId());
            $orderId = $result['orderId'] ? $result['orderId'] : 0;

            if ($orderId) {
                return ['magento order has been processed: ' . $orderId];
            }
            
            return ['magento order could not processed'];
        }

        if ($response['event'] == ConstantValue::WEBHOOK_EVENT_ORDER_AUTHORISED) {
            return ['magento order already processed'];
        }

        $magentoOrder = $this->orderInterface->loadByIncrementId($this->revolutOrder->getIncrementOrderId());
        
        if (!$magentoOrder) {
            return ['order not found'];
        }

        $magentoPayment = $magentoOrder->getPayment();

        if (!$magentoPayment) {
            return ['order payment not found'];
        }

        $webhookAction = $magentoPayment->getAdditionalInformation(ConstantValue::WEBHOOK_ACTION);
        switch ($webhookAction) {
            case ConstantValue::WEBHOOK_ACTION_ACCEPT_PAYMENT:
                $magentoPayment->accept();
                $magentoOrder->save();
                $this->logger->debug('magento payment accepted: ' . json_encode($response));
                return ['success'];
            case ConstantValue::WEBHOOK_ACTION_CAPTURE_PAYMENT:
                $magentoPayment->capture();
                $magentoOrder->save();
                $this->logger->debug('magento payment captured: ' . json_encode($response));
                return ['success'];
            case ConstantValue::WEBHOOK_ACTION_PAYMENT_COMPLETED:
                $this->logger->debug('magento payment already completed: ' . json_encode($response));
                return ['success'];
        }

        $this->logger->debug('webhook action not found: ' . json_encode($response));
        return ['webhook action not found'];
    }

    /**
     * Process Magento Order
     *
     * @param PaymentInterface $paymentMethod
     * @return array.
     * @api
     */
    public function placeOrder(PaymentInterface $paymentMethod)
    {
        try {
            $additionalData = $paymentMethod->getAdditionalData();
            $publicId = isset($additionalData['publicId']) ? $additionalData['publicId'] : '';
            $setAgreement = isset($additionalData['setAgreement']) ? (bool)$additionalData['setAgreement'] : false;

            if (empty($publicId)) {
                throw new LocalizedException(__('publicId can not be empty'));
            }

            $this->revolutOrder = $this->revolutOrder->retrieveBy('public_id', $publicId);
            
            $orderProcessingResult = [];

            if (empty($this->revolutOrder->getIncrementOrderId())) {
                $orderProcessingResult = $this->revolutPaymentInformationManagement->placeOrder($publicId, $paymentMethod, $setAgreement);
                $this->revolutOrder = $this->revolutOrder->retrieveBy('public_id', $publicId);
            }
            
            if (empty($this->revolutOrder->getIncrementOrderId())) {
                $errorMsg = !empty($orderProcessingResult['errorMsg']) ? $orderProcessingResult['errorMsg']: 'Something went wrong.';
                throw new LocalizedException(__($errorMsg));
            }

            $magentoOrder = $this->orderInterface->loadByIncrementId($this->revolutOrder->getIncrementOrderId());
            $this->checkoutSession->setQuoteId($this->revolutOrder->getQuoteId());
            $this->checkoutSession->setLastQuoteId($this->revolutOrder->getQuoteId());
            $this->checkoutSession->setLastSuccessQuoteId($this->revolutOrder->getQuoteId());
            $this->checkoutSession->setLastOrderId($magentoOrder->getId());
            $this->checkoutSession->setLastRealOrderId($magentoOrder->getIncrementId());
            $this->checkoutSession->setLastOrderStatus($magentoOrder->getStatus());
                
            $store = $this->storeManager->getStore($this->revolutOrder->getStoreId());
            $quote = $this->quoteFactory->create()->load($this->revolutOrder->getQuoteId());

            $this->eventManager->dispatch('checkout_submit_all_after', ['order' => $magentoOrder, 'quote' => $quote]);
                
            $result = [
                'success' => true,
                'redirectUrl' =>  $store->getBaseUrl() . 'checkout/onepage/success'
            ];
        } catch (\Exception $e) {
            $this->logger->debug('placeOrder Exception: ' . $e->getMessage());
            $result = [
                'success' => false,
                'message' => $e->getMessage()
            ];
        }

        return [$result];
    }

    /**
     * Cancel order on revolut
     *
     * @return array.
     * @api
     */
    public function cancel()
    {
        try {
            $publicId = $this->getPublicIdParam();

            $this->revolutOrder->cancel($publicId);

            $result = [
                'success' => true,
            ];
        } catch (\Exception $e) {
            $this->logger->debug($e->getMessage());
            $result = [
                'success' => false,
                'mess' => 'An error occurred while canceling the order'
            ];
        }

        return [$result];
    }
}
