<?php

namespace Revolut\Payment\Model\Helper\Api;

use Revolut\Payment\Gateway\Helper\AmountProvider;
use Magento\Framework\Json\Helper\Data;
use Magento\Framework\App\Helper\Context;
use Revolut\Payment\Model\Helper\ConstantValue;
use Revolut\Payment\Gateway\Config\Config;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\Module\ModuleListInterface;
use Magento\Framework\App\ProductMetadataInterface;
use Revolut\Payment\Model\Helper\Logger;

/**
 * Class RevolutApi
 */
class RevolutApi extends AbstractHelper
{
    /**
     * @var ModuleListInterface
     */
    protected $moduleList;

    /**
     * @var ProductMetadataInterface
     */
    protected $productMetadata;

    /**
     * @var Config
     */
    protected $config;

    /**
     * @var Logger
     */
    protected $logger;

    /**
     * @var Data
     */
    protected $jsonHelper;
    
    /**
     * @var AmountProvider
     */
    protected $amountProvider;

    /**
     * RevolutApi constructor.
     *
     * @param Config $config
     * @param Data $jsonHelper
     * @param Context $context
     * @param Logger $logger
     * @param AmountProvider $amountProvider
     * @param ModuleListInterface $moduleList
     * @param ProductMetadataInterface $productMetadata
     */
    public function __construct(
        Config $config,
        Data $jsonHelper,
        Context $context,
        Logger $logger,
        AmountProvider $amountProvider,
        ModuleListInterface $moduleList,
        ProductMetadataInterface $productMetadata
    ) {
        parent::__construct($context);
        $this->config = $config;
        $this->jsonHelper = $jsonHelper;
        $this->logger = $logger;
        $this->moduleList = $moduleList;
        $this->amountProvider = $amountProvider;
        $this->productMetadata = $productMetadata;
    }

    /**
     * @param string $url
     * @param int $storeId
     * @param string $method
     * @param array|null $content
     * @return array
     * @throws \Exception
     */
    public function request($url, $storeId, $method, $content = null)
    {
        $httpHeaders = new \Zend\Http\Headers();
        $setup_version =  $this->moduleList->getOne(ConstantValue::MODULE_NAME)['setup_version']; // @phpstan-ignore-line
        $httpHeaders->addHeaders(
            array_merge(
                [
                "Authorization" => "Bearer " . $this->config->getApiKey($storeId),
                "User-Agent" => 'Revolut Payment Gateway/' . $setup_version
                . ' Magento/' . $this->productMetadata->getVersion()
                . ' PHP/' . PHP_VERSION,
                "Content-Type" => "application/json; charset=utf-8"
                ]
            )
        );

        $content = $content ? $this->jsonHelper->jsonEncode($content) : $content;

        $request = new \Zend\Http\Request();
        $request->setHeaders($httpHeaders);
        $request->setContent($content);
        $request->setUri($url);
        $request->setMethod($method);

        $client = new \Zend\Http\Client();
        $options = [
            'adapter' => 'Zend\Http\Client\Adapter\Curl',
            'curloptions' => [CURLOPT_FOLLOWLOCATION => true],
            'maxredirects' => 0,
            'timeout' => 60
        ];
        $client->setOptions($options);
        try {
            $response = $client->send($request);
            if ($response->getStatusCode() >= 400 && $response->getStatusCode() < 500 && 'GET' !== $method) {
                throw new \Exception("Something went wrong: $method $url\n" . $response->getBody());
            }
            return $this->json($response->getBody());
        } catch (\Exception $e) {
            $this->logger->debug("API error: " . $e->getMessage());
            throw new \Exception(__("API error: " . $e->getMessage()));
        }
    }

    /**
     * @param  string $url
     * @param  int $storeId
     * @param  array|null $content
     * @return array
     */
    public function post($url, $storeId, $content = null)
    {
        return $this->request($url, $storeId, 'POST', $content);
    }
    
    /**
     * @param  string $url
     * @param  int $storeId
     * @return array
     */
    public function get($url, $storeId)
    {
        return $this->request($url, $storeId, 'GET');
    }
    
    /**
     * @param  string $url
     * @param  int $storeId
     * @param  array|null $content
     * @return array
     */
    public function patch($url, $storeId, $content = null)
    {
        return $this->request($url, $storeId, 'PATCH', $content);
    }
    
    /**
     * @param  string     $url
     * @param  int        $storeId
     * @param  array|null $content
     * @return array
     */
    public function put($url, $storeId, $content = null)
    {
        return $this->request($url, $storeId, 'PUT', $content);
    }
    
    /**
     * @param  string $url
     * @param  int $storeId
     * @return array
     */
    public function delete($url, $storeId)
    {
        return $this->request($url, $storeId, 'DELETE');
    }

    /**
     * @param  string $response
     * @return array
     */
    public function json($response)
    {
        return (array)$this->jsonHelper->jsonDecode($response);
    }
    
    /**
     * @param  int $storeId
     * @return string
     */
    public function getMerchantPublicKey($storeId)
    {
        $publicKeyUrl = $this->config->getMgmtApiUrl($storeId) . ConstantValue::API . '/' . ConstantValue::ENDPOINT_PUBLIC_KEY;
        $response = $this->get($publicKeyUrl, $storeId);
        return isset($response['public_key']) ?  (string)$response['public_key'] : '';
    }
    
    /**
     * @param int $storeId
     * @param string $publicId
     * @return array
     */
    public function getAvailableCardBrands($storeId, $publicId)
    {
        $response = $this->get($this->config->getMgmtApiUrl($storeId) . '/' . ConstantValue::ENDPOINT_ORDER . '/token/' . $publicId, $storeId);
        return isset($response['availableCardBrands']) ? $response['availableCardBrands'] : [];
    }

    /**
     * @param string $domain
     * @param int $storeId
     * @return array
     */
    public function registerApplePayDomain($domain, $storeId)
    {
        $url = $this->config->getApiUrl($storeId) . '/api/apple-pay/domains/register';
        return $this->post($url, $storeId, ["domain" => $domain]);
    }
}
